"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyOpenUrlSetting = void 0;
const colors_1 = __importDefault(require("colors"));
const yargs_1 = require("yargs");
const url_1 = require("url");
const helpers_1 = require("../webpack/helpers");
const applyOpenUrlSetting = (config, settings) => {
    const configName = yargs_1.argv['config'];
    if (configName) {
        const serveConfig = helpers_1.getJSONFile('config/serve.json');
        if (serveConfig.serveConfigurations[configName]) {
            const configValue = serveConfig.serveConfigurations[configName];
            const openUrl = new url_1.URL(configValue.pageUrl);
            openUrl.searchParams.set('debugManifestsFile', 'https://localhost:4321/temp/manifests.js');
            if (configValue.customActions) {
                openUrl.searchParams.set('loadSPFX', 'true');
                openUrl.searchParams.set('customActions', JSON.stringify(configValue.customActions));
            }
            if (configValue.fieldCustomizers) {
                openUrl.searchParams.set('loadSPFX', 'true');
                openUrl.searchParams.set('fieldCustomizers', JSON.stringify(configValue.fieldCustomizers));
            }
            config.devServer.open = true;
            config.devServer.openPage = openUrl.href;
            helpers_1.logDebugString(`Loading ${colors_1.default.yellow(configName)} serve configuration and opening ${colors_1.default.green(openUrl.href)}`);
        }
        else {
            throw new Error(`Unable to find serve configuration with name '${configName}' in serve.json`);
        }
        return;
    }
    if (!settings.openUrl) {
        config.devServer.open = false;
    }
    else {
        config.devServer.open = true;
        config.devServer.openPage = settings.openUrl;
    }
};
exports.applyOpenUrlSetting = applyOpenUrlSetting;
//# sourceMappingURL=openUrl.js.map