"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createBaseConfig = void 0;
const webpack_1 = __importDefault(require("webpack"));
const path = __importStar(require("path"));
const certificateManager = require('@rushstack/debug-certificate-manager');
const certificateStore = new certificateManager.CertificateStore();
const AsyncComponentPlugin_1 = require("@microsoft/spfx-heft-plugins/lib/plugins/webpackConfigurationPlugin/webpackPlugins/AsyncComponentPlugin");
const fork_ts_checker_webpack_plugin_1 = __importDefault(require("fork-ts-checker-webpack-plugin"));
const ClearCssModuleDefinitionsPlugin_1 = require("../plugins/ClearCssModuleDefinitionsPlugin");
const TypeScriptResourcesPlugin_1 = require("../plugins/TypeScriptResourcesPlugin");
const helpers_1 = require("./helpers");
const tsconfig_paths_webpack_plugin_1 = __importDefault(require("tsconfig-paths-webpack-plugin"));
const tsconfig_1 = require("tsconfig");
const rootFolder = path.resolve(process.cwd());
function createBaseConfig(cli) {
    return __awaiter(this, void 0, void 0, function* () {
        let port = 0;
        if (!cli.isLibraryComponent) {
            port = 4321;
        }
        else {
            if (!cli.port) {
                port = 4320;
            }
            else {
                port = cli.port;
            }
        }
        yield helpers_1.freePortIfInUse(port);
        const host = 'https://localhost:' + port;
        const cssLoader = require.resolve('css-loader');
        const themedLoader = {
            loader: require.resolve('@microsoft/loader-load-themed-styles'),
            options: {
                async: true
            }
        };
        const cleanCssLoader = {
            loader: require.resolve('clean-css-loader'),
            options: {
                level: {
                    1: {
                        all: false,
                        removeQuotes: true
                    }
                }
            }
        };
        const externalComponents = helpers_1.getExternalComponents();
        const baseConfig = {
            target: 'web',
            mode: 'development',
            devtool: 'source-map',
            resolve: {
                extensions: ['.ts', '.tsx', '.js'],
                modules: ['node_modules']
            },
            context: rootFolder,
            module: {
                rules: [
                    {
                        use: [
                            {
                                loader: require.resolve('ts-loader'),
                                options: {
                                    transpileOnly: true,
                                    compilerOptions: {
                                        declarationMap: false
                                    }
                                },
                            }
                        ],
                        test: /\.tsx?$/,
                        exclude: /node_modules/
                    },
                    {
                        use: [{
                                loader: require.resolve('file-loader'),
                                options: {
                                    name: '[name]_[hash].[ext]',
                                    esModule: false
                                }
                            }],
                        test: /\.(jpe?g|png|woff|eot|ttf|svg|gif|dds)$/i
                    },
                    {
                        use: [{
                                loader: 'html-loader'
                            }],
                        test: /\.html$/
                    },
                    {
                        test: /\.css$/,
                        use: [
                            themedLoader,
                            {
                                loader: cssLoader,
                                options: {
                                    esModule: false,
                                    modules: false
                                }
                            }
                        ]
                    },
                    {
                        test: function (fileName) {
                            return fileName.endsWith('.module.scss');
                        },
                        use: [
                            themedLoader,
                            require.resolve('spfx-css-modules-typescript-loader'),
                            {
                                loader: cssLoader,
                                options: {
                                    esModule: false,
                                    modules: {
                                        localIdentName: '[local]_[hash:base64:8]',
                                        mode: (resourcePath) => {
                                            if (resourcePath.endsWith('.css') && /node_modules/gi.test(resourcePath)) {
                                                return 'global';
                                            }
                                            return 'local';
                                        }
                                    }
                                }
                            },
                            cleanCssLoader,
                            {
                                loader: require.resolve('sass-loader'),
                                options: {
                                    implementation: require('node-sass')
                                }
                            }
                        ]
                    },
                    {
                        test: function (fileName) {
                            return !fileName.endsWith('.module.scss') && !fileName.endsWith('.vue.scss') && fileName.endsWith('.scss');
                        },
                        use: [
                            themedLoader,
                            {
                                loader: cssLoader,
                                options: {
                                    esModule: false,
                                    modules: false
                                }
                            },
                            cleanCssLoader,
                            {
                                loader: require.resolve('sass-loader'),
                                options: {
                                    implementation: require('node-sass')
                                }
                            }
                        ]
                    }
                ]
            },
            plugins: [
                new AsyncComponentPlugin_1.AsyncComponentPlugin({
                    externalComponents
                }),
                new webpack_1.default.WatchIgnorePlugin([path.resolve(rootFolder, 'temp')]),
                new fork_ts_checker_webpack_plugin_1.default({
                    async: true
                }),
                new TypeScriptResourcesPlugin_1.TypeScriptResourcesPlugin(),
                new ClearCssModuleDefinitionsPlugin_1.ClearCssModuleDefinitionsPlugin({
                    rootFolder
                }),
                new webpack_1.default.DefinePlugin({
                    'process.env.NODE_ENV': JSON.stringify(process.env.NODE_ENV),
                    'DEBUG': JSON.stringify(true)
                })
            ],
            devServer: {
                hot: false,
                contentBase: rootFolder,
                publicPath: host + '/dist/',
                host: 'localhost',
                port: port,
                disableHostCheck: true,
                historyApiFallback: true,
                writeToDisk: cli.isLibraryComponent,
                headers: {
                    'Access-Control-Allow-Origin': '*',
                },
                https: {
                    cert: certificateStore.certificateData,
                    key: certificateStore.keyData
                }
            },
        };
        const tsConfigPath = path.resolve(rootFolder, 'tsconfig.json');
        const tsconfig = yield tsconfig_1.readFile(tsConfigPath);
        if (tsconfig.baseUrl) {
            baseConfig.resolve.plugins = [
                new tsconfig_paths_webpack_plugin_1.default({ configFile: tsConfigPath })
            ];
        }
        return baseConfig;
    });
}
exports.createBaseConfig = createBaseConfig;
//# sourceMappingURL=baseConfig.js.map