"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.logDebugString = exports.addCopyLocalExternals = exports.createLocalExternals = exports.getExternalComponents = exports.checkVersions = exports.freePortIfInUse = exports.createResourcesMap = exports.createKeyFromPath = exports.trim = exports.trimRight = exports.trimLeft = exports.addCopyLocalizedResources = exports.getEntryPoints = exports.getLoggingLevel = exports.setDefaultServeSettings = exports.getJSONFile = void 0;
const path = __importStar(require("path"));
const fs = __importStar(require("fs"));
const globby = __importStar(require("globby"));
const get_port_1 = __importDefault(require("get-port"));
const colors_1 = __importDefault(require("colors"));
const killPort = require('kill-port');
function getJSONFile(relPath) {
    return require(path.join(process.cwd(), relPath));
}
exports.getJSONFile = getJSONFile;
function setDefaultServeSettings(settings) {
    const defaultServeSettings = {
        eslint: false,
        fullScreenErrors: true,
        loggingLevel: 'normal',
        hotRefresh: false,
        openUrl: undefined,
        reactProfiling: false,
        containers: undefined
    };
    settings.serve = settings.serve || {};
    settings.serve = Object.assign(defaultServeSettings, settings.serve);
    if (settings.cli.isLibraryComponent) {
        settings.serve.openUrl = undefined;
    }
}
exports.setDefaultServeSettings = setDefaultServeSettings;
function getLoggingLevel(level) {
    if (level === 'minimal') {
        return {
            all: false,
            colors: true,
            errors: true
        };
    }
    if (level === 'normal') {
        return {
            all: false,
            colors: true,
            errors: true,
            warnings: true,
            timings: true,
            entrypoints: true
        };
    }
    if (level === 'detailed') {
        return {
            all: false,
            colors: true,
            errors: true,
            timings: true,
            assets: true,
            warnings: true
        };
    }
    throw new Error('Unsupported log level: ' + level);
}
exports.getLoggingLevel = getLoggingLevel;
function getEntryPoints(entry) {
    const newEntry = {};
    for (const key in entry) {
        let entryPath = entry[key];
        if (entryPath.indexOf('bundle-entries') === -1) {
            entryPath = createTsEntryPath(entryPath);
        }
        else {
            let bundleContent = fs.readFileSync(entryPath).toString();
            bundleContent = createTsEntriesForBundledPackage(bundleContent);
            fs.writeFileSync(entryPath, bundleContent);
        }
        newEntry[key] = entryPath;
    }
    return newEntry;
}
exports.getEntryPoints = getEntryPoints;
function createTsEntriesForBundledPackage(content) {
    const search = /require\('(?<jsPath>.*)'\)/gi;
    let newContent = content.slice();
    let match = search.exec(content);
    do {
        const jsPath = match.groups.jsPath;
        const tsPath = createTsEntryPath(jsPath);
        newContent = newContent.replace(jsPath, tsPath);
    } while ((match = search.exec(content)) !== null);
    return newContent;
}
function createTsEntryPath(jsPath) {
    const libSearchRegexp1 = /\/lib\//gi;
    const libSearchRegexp2 = /\\lib\\/gi;
    const srcPathToReplace1 = '/src/';
    const srcPathToReplace2 = '\\src\\';
    const tsPath = jsPath
        .replace(libSearchRegexp1, srcPathToReplace1)
        .replace(libSearchRegexp2, srcPathToReplace2)
        .slice(0, -3) + '.ts';
    return getEntryPath(tsPath);
}
function getEntryPath(tsPath) {
    let pathToCheck = tsPath;
    if (!path.isAbsolute(pathToCheck)) {
        const bundledEntriesPath = path.join(process.cwd(), 'temp/bundle-entries');
        pathToCheck = path.join(bundledEntriesPath, pathToCheck);
    }
    if (fs.existsSync(pathToCheck)) {
        return tsPath;
    }
    pathToCheck = pathToCheck + 'x';
    if (fs.existsSync(pathToCheck)) {
        return tsPath + 'x';
    }
    throw new Error('Unable to resolve entry path. Path received: ' + tsPath);
}
function addCopyLocalizedResources(localizedResources) {
    const patterns = [];
    for (const resourceKey in localizedResources) {
        const resourcePath = localizedResources[resourceKey];
        const from = resourcePath.replace(/^lib/gi, 'src').replace('{locale}', '*');
        patterns.push({
            flatten: true,
            from,
            noErrorOnMissing: true,
            to: (data) => {
                let fileName = path.basename(data.absoluteFilename);
                if (!resourcePath.endsWith('{locale}.js')) {
                    fileName = 'default.js';
                }
                return resourceKey + '_' + fileName;
            }
        });
    }
    return patterns;
}
exports.addCopyLocalizedResources = addCopyLocalizedResources;
function trimLeft(str, charlist) {
    return str.replace(new RegExp('^[' + charlist + ']+'), '');
}
exports.trimLeft = trimLeft;
function trimRight(str, charlist) {
    return str.replace(new RegExp('[' + charlist + ']+$'), '');
}
exports.trimRight = trimRight;
function trim(str, charlist) {
    return trimRight(trimLeft(str, charlist), charlist);
}
exports.trim = trim;
function createKeyFromPath(path) {
    return trimLeft(path, '/\\\\').replace(/\//gi, '|').replace(/\\/gi, '|');
}
exports.createKeyFromPath = createKeyFromPath;
function createResourcesMap(localizedResources) {
    const resourcesMap = {};
    for (const resourceKey in localizedResources) {
        const resourcePath = localizedResources[resourceKey];
        const search = resourcePath.replace(/^lib/gi, 'src').replace('{locale}.js', '*.ts');
        const exclude = '!' + search.replace('*.ts', '*.d.ts');
        const typescriptResources = globby.sync([search, exclude], {
            cwd: process.cwd()
        });
        if (!(typescriptResources === null || typescriptResources === void 0 ? void 0 : typescriptResources.length)) {
            continue;
        }
        for (const resourcePath of typescriptResources) {
            const key = createKeyFromPath(resourcePath);
            const fileName = `${resourceKey}_${path.basename(resourcePath).replace('.ts', '.js')}`;
            resourcesMap[key] = {
                path: trimLeft(resourcePath, '/\\\\'),
                fileName
            };
        }
    }
    return resourcesMap;
}
exports.createResourcesMap = createResourcesMap;
function freePortIfInUse(port) {
    return __awaiter(this, void 0, void 0, function* () {
        const freePort = yield get_port_1.default({ port, host: 'localhost' });
        if (freePort !== port) {
            console.log(colors_1.default.yellow(`The port ${port} is in use. Trying to release...`));
            yield killPort(port);
            console.log(colors_1.default.yellow(`The port ${port} is successfully released.`));
        }
    });
}
exports.freePortIfInUse = freePortIfInUse;
function checkVersions() {
    const packageJson = getJSONFile('package.json');
    const spfxVersion = getMinorVersion(packageJson, '@microsoft/sp-build-web');
    const fastServeVersion = getMinorVersion(packageJson, 'spfx-fast-serve-helpers');
    if (spfxVersion !== fastServeVersion) {
        throw new Error(`SPFx Fast Serve: version mismatch. We detected the usage of SPFx 1.${spfxVersion}, but "spfx-fast-serve-helpers" version is 1.${fastServeVersion}. Please change "spfx-fast-serve-helpers" version to ~1.${spfxVersion}.0, delete node_modules, package-lock.json and reinstall dependencies.`);
    }
}
exports.checkVersions = checkVersions;
function getMinorVersion(packageJson, dependecyToCheck) {
    let version = packageJson.devDependencies[dependecyToCheck];
    if (!version) {
        version = packageJson.dependencies[dependecyToCheck];
    }
    if (!version) {
        throw new Error(`SPFx Fast Serve: unable to find dependency ${dependecyToCheck}`);
    }
    if (version.indexOf('~') === 0 || version.indexOf('^') === 0) {
        version = version.substr(1);
    }
    return parseInt(version.split('.')[1]);
}
function getExternalComponents() {
    const originalWebpackConfig = getJSONFile('temp/_webpack_config.json');
    for (const plugin of originalWebpackConfig.plugins) {
        if (plugin._options && plugin._options.externalComponents) {
            return plugin._options.externalComponents;
        }
    }
    throw new Error('Unable to resolve AsyncComponentPlugin');
}
exports.getExternalComponents = getExternalComponents;
function createLocalExternals(externals) {
    if (!externals)
        return null;
    const result = {};
    for (const name in externals) {
        const info = externals[name];
        if (typeof info === 'string') {
            if (info.startsWith('http')) {
                continue;
            }
            result[name] = { path: info };
        }
        else {
            if (info.path.startsWith('http')) {
                continue;
            }
            result[name] = { path: info.path };
        }
    }
    return result;
}
exports.createLocalExternals = createLocalExternals;
function addCopyLocalExternals(externals, manifest, originalEntries) {
    if (!externals)
        return [];
    const patterns = [];
    for (const jsModule of manifest) {
        if (jsModule.loaderConfig
            && jsModule.loaderConfig.entryModuleId
            && originalEntries.indexOf(jsModule.loaderConfig.entryModuleId) !== -1) {
            for (const resourceKey in jsModule.loaderConfig.scriptResources) {
                const resource = jsModule.loaderConfig.scriptResources[resourceKey];
                if (externals[resourceKey]) {
                    const from = externals[resourceKey].path;
                    const to = resource.path;
                    if (!hasPattern(patterns, to)) {
                        patterns.push({
                            flatten: true,
                            from,
                            noErrorOnMissing: true,
                            to
                        });
                    }
                }
            }
        }
    }
    return patterns;
}
exports.addCopyLocalExternals = addCopyLocalExternals;
function hasPattern(patterns, to) {
    for (const pattern of patterns) {
        if (pattern.to === to) {
            return true;
        }
    }
    return false;
}
function logDebugString(message) {
    console.log(`${getTimeString()} [${colors_1.default.cyan('fast-serve')}] ${message}`);
}
exports.logDebugString = logDebugString;
function getTimeString() {
    const now = new Date();
    return `[${colors_1.default.gray(`${('0' + now.getHours()).slice(-2)}:${('0' + now.getMinutes()).slice(-2)}:${('0' + now.getSeconds()).slice(-2)}`)}]`;
}
//# sourceMappingURL=helpers.js.map